import csv
import os
import time
import string
import sc

# number of values to parse
numItems = 14
# number of beat slots
numRows = 16
#input file
fileName = "input.csv"
#time to wait (duration of a 16th note)
waitTime = 1.0/8.0
# initial state
state = {
    "Hat Len": 1,
    "Hat Delay": 0,
    "Bass Freq": 110,
    "Pling Freq": 440,
    "Pling Att": 0.001,
    "Pling Dec": 0.4,
    "Pling Delay": 0
}

# SuperCollider stuff (and effect nodes)
scsynth = sc.Server("127.0.0.1", 57110)

hatDelayIn = 64
hatDelay = sc.Synth(scsynth, "fbDelay", 1, {
    "in": hatDelayIn,
    "delayTime": 0.25,
    "feedback": 0.5
})

plingDelayIn = 66
plingDelay = sc.Synth(scsynth, "fbDelay", 1, {
    "in": plingDelayIn,
    "delayTime": 1/3.0,
    "feedback": 0.5
})


def sanitizeCell(input):
    input = input.upper()
    input = string.replace(input, ",", ".")
    if input == "X": input = 1.0
    try:
        output = float(input)
    except Exception, e:
        output = 0
    
    return output

def readSpreadsheet(file):
    
    data = csv.reader(open(file, 'rb'))
    headings = data.next()[1:numItems+1]
    
    list = []
    
    for i in range(numRows):
        values = data.next()[1:numItems+1]
        dict = {}
        for e in range(numItems):
            if values[e] != "":
                dict[headings[e]] = sanitizeCell(values[e])
        
        list.append(dict)
    
    return list

beatz = readSpreadsheet(fileName)
lastUpdate = time.time()

def tryUpdateBeatz(file):
    global lastUpdate
    global beatz
    fileModTime = os.path.getmtime(file)
    if fileModTime > lastUpdate:
        beatz = readSpreadsheet(file)
        lastUpdate = time.time()
        
# the main music function
def processRow(row):
    # update state
    for key, value in state.iteritems():
        if key in row:
            state[key] = row[key]
    
    # now up to some spagetthi code!
    if "Kick (Amp)" in row:
        sc.Synth(scsynth, "kick", 0, {
            "amp": row["Kick (Amp)"]
        })
        
    if "Hat (Amp)" in row:
        sc.Synth(scsynth, "hat", 0, {
            "amp": row["Hat (Amp)"],
            "len": state["Hat Len"],
            "delayBus": hatDelayIn,
            "delay": state["Hat Delay"]
        })
    
    if "Snare (Amp)" in row:
        sc.Synth(scsynth, "snare", 0, {
            "amp": row["Snare (Amp)"]
        })
    
    if "Bass (Amp)" in row:
        sc.Synth(scsynth, "bass", 0, {
            "amp": row["Bass (Amp)"],
            "freq": state["Bass Freq"]
        })
        
    if "Pling (Amp)" in row:
        sc.Synth(scsynth, "pling", 0, {
            "amp": row["Pling (Amp)"],
            "freq": state["Pling Freq"],
            "att": state["Pling Att"],
            "dec": state["Pling Dec"],
            "delayBus": plingDelayIn,
            "delay": state["Pling Delay"]
        })
    
    if "Hat D-Time" in row:
        hatDelay.set("delayTime", row["Hat D-Time"])
        
    if "Pling D-Time" in row:
        plingDelay.set("delayTime", row["Pling D-Time"])

# main loop
currentIndex = 0
while True:

    currentRow = beatz[currentIndex]
    processRow(currentRow)
    
    currentIndex = currentIndex + 1
    if currentIndex >= numRows:
        currentIndex = 0
    time.sleep(waitTime)
    tryUpdateBeatz(fileName)
